﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace CustomPropertyGrid.PropertyEditors
{
	public partial class EnumPropertyEditor : PropertyEditor
	{
		private	bool	updatingFromObj	= false;

		public override string PropertyName
		{
			get { return this.nameLabel.Text; }
			set { this.nameLabel.Text = value; }
		}
		public override object DisplayedValue
		{
			get { return Convert.ChangeType(this.valueEditor.EnumValue, this.EditedType); }
		}

		public EnumPropertyEditor(PropertyEditor parentEditor, PropertyGrid parentGrid) : base(parentEditor, parentGrid)
		{
			this.InitializeComponent();
		}

		public override void PerformGetValue()
		{
			base.PerformGetValue();
			object[] values = this.Getter().ToArray();

			this.updatingFromObj = true;
			if (!values.Any())
				this.valueEditor.EnumValue = (Enum)ReflectionHelper.CreateInstanceOf(this.EditedType);
			else
			{
				Enum firstVal = (Enum)values.Where(o => o != null).First();

				this.valueEditor.EnumValue = firstVal;

				if (!this.ReadOnly && (values.Any(o => o == null) || !values.All(o => Enum.Equals(o, firstVal))))
					this.valueEditor.BackColor = this.BackColorMultiple;
				else
					this.valueEditor.BackColor = this.BackColorDefault;
			}
			this.updatingFromObj = false;
		}
		public override void PerformSetValue()
		{
			base.PerformSetValue();
			if (this.ReadOnly) return;

			this.SetterSingle(this.DisplayedValue);
		}
		public override void UpdateReadOnlyState()
		{
		    base.UpdateReadOnlyState();
			this.valueEditor.Enabled = !this.ReadOnly;
		}

		protected override void OnEditedTypeChanged()
		{
			base.OnEditedTypeChanged();
			this.updatingFromObj = true;
			this.valueEditor.EnumValue = (Enum)ReflectionHelper.CreateInstanceOf(this.EditedType);
			this.updatingFromObj = false;
		}
		protected override void OnSizeChanged(EventArgs e)
		{
			base.OnSizeChanged(e);
			this.nameLabel.Width = this.NameLabelWidth;
		}

		private void valueEditor_EnumValueChanged(object sender, EventArgs e)
		{
			if (this.updatingFromObj) return;
			this.PerformSetValue();
			this.OnValueEdited(this.DisplayedValue);
			this.PerformGetValue();
		}
	}
}
